@@ -11,11 +11,11 @@ __all__ = [
     "TimeBasedMovement",
     "updown",
 ]
-__version__ = "0.1.9"
+__version__ = "0.1.13"
 
 from .aia_utilities import (
-    Redis_Utilities,
+    RedisUtilities,
     TimeBasedMovement,
-    Time_Management,
+    TimeManagement,
     Helpers
 )
